require 'net/http'
require 'json'

Shoes.setup do
  gem 'net_http_ssl_fix'#, require: 'net_http_ssl_fix'
end

DEFAULT_URL = ''
DEFAULT_USERNAME = ''
DEFAULT_PASSWORD = ''

if File.exists?('config.json')
  config = JSON.parse(File.read('config.json'))
  DEFAULT_URL = config['url']
  DEFAULT_USERNAME = config['username']
  DEFAULT_PASSWORD = config['password']
end


def valid_json?(json)
  begin
    JSON.parse(json)
    return true
  rescue Exception => e
    return false
  end
end

def reset_text(items)
  items.each do |item|
    item.text = nil
  end
end

def reset_select(items)
  items.each do |item|
    item.choose('')
  end
end

def enable(items)
  items.each do |item|
    item.state = nil
  end
end

def disable(items)
  items.each do |item|
    item.state = 'disabled'
  end
end

def label_style
 { size: 10, align: 'right', stroke: "#FFFFFF" }
end

def construct_url
  reset_text [@response, @status, @json]
  
  @id.text = '' unless [:show, :update, :delete].include? @action.text

  text = @url.text + '/'
  text += @items.text + '/' if @items.text != ''
  text += @id.text + '/' if @id.text != ''

  @uri.text = text
end

def construct_params
  
  @uri.text = @uri.text.split('?')[0]
  
  url_params = []
  url_params << :page if @page.text != ''
  url_params << :params if @params.text != ''
  
  @uri.text += '?' if url_params.count > 0

  url_params.each do |url_param|
    case url_param
      when :page
        @uri.text += "page=#{@page.text.to_s}"
      when :params
        @uri.text += "#{@params.text.to_s}"
    end
    
    @uri.text += '&' unless url_param == url_params.last
  end

end

Shoes.app title: 'Changeworknow API', width: 1440, height: 840 do
 
  background "#282a45"

  
  @modules = {
     '' => [],
     applications: ['', :index, :show],
     business_units: ['', :index, :show],
     candidates: ['', :index, :show],
     configuration_scopes: ['' , :index, :show],
     employees: ['', :index, :show, :create, :update, :delete],
     employments: ['', :index, :show, :create, :update, :delete],     
     organizational_sub_units: ['', :index, :show],
     organizational_units: ['', :index, :show],
     position_opening_templates: ['', :index, :show],
     position_openings: ['', :index, :show, :create, :update, :delete], 
     position_reasons: ['', :index, :show],
     position_requisition_templates: ['', :index, :show],
     position_requisitions: ['', :index, :show, :create, :update, :delete],
     remuneration_package_keywords: ['', :index, :show],
     shift_keywords: ['', :index, :show],  
     termination_reasons: ['', :index, :show],     
     user_groups: ['', :index, :show],
     worksites: ['', :index, :show, :create, :update, :delete],
     custom: []
  }

  flow margin: 10, width: 640 do
    stack margin: 10 do
      flow width: 600 do
        flow width: 100 do
          para "URL: ", label_style
        end
               
        @url = list_box width: 500, items: ['https://gateway.changeworknow.co.uk/hr_api/v1', 'https://gateway-staging.changeworknow.co.uk/hr_api/v1', 'http://localhost:3000/hr_api/v1'], change: 'e' do |item|
          reset_select [@items, @action]         
          disable [@id, @action, @button]
          
          construct_url          
        end        

      end

      flow width: 600 do
        flow width: 100 do
          para "Username: ", label_style
        end
        @username = edit_line text: DEFAULT_USERNAME, width: 500
      end
      
      flow width: 600 do
        flow width: 100 do
          para "Password: ", label_style
        end
        @password = edit_line text: DEFAULT_PASSWORD, width: 500
      end
      
      flow width: 600 do
        @connection = button "Test Connection", width: 600, height: 50 do
        
          uri = URI("#{@url.text}/user_groups")
          http = Net::HTTP.new(uri.host, uri.port)   

          req = Net::HTTP::Get.new(uri)             

          if @url.text.include?('https')
            http.use_ssl = true      
            http.verify_mode = OpenSSL::SSL::VERIFY_NONE
          end

          req['Username'] = @username.text
          req['Password'] = @password.text
          req['Content-Type'] = 'application/json'     


          res = http.request(req) rescue 404

          if res == 404
            sense = { api: 'Not found', cms: 'Not found', code: 404}
          else
            sense = case res.code.to_i
              when 200
                { api: 'OK', cms: 'OK', code: 200}
              when 404
                { api: 'OK', cms: 'Not found', code: 404 }
              when 401
                { api: 'Unauthorized', cms: 'Unauthorized', code: 401 }
              when 302
                { api: 'OK', cms: 'Unauthorized', code: "401/302" }
              else
                { api: 'None', cms: 'None', code: 404 }
            end
          end
          
          string = "Connection to API: #{sense[:api]} \n\nConnection to CMS: #{sense[:cms]}\n\n\n#{sense[:code]}"
        
          alert(string)
        end
      end
      
      flow width: 600 do
        flow width: 100 do
          para "Items: ", label_style
        end
        @items = list_box width: 250, items: @modules.keys.map { |x| x.to_s }, change: 'e' do |item|
          construct_url
                     
          reset_select [@action]
          disable [@id, @button, @uri, @method, @json]
          
          if item.text == ''
            disable [@action]
          else
            if item.text == 'custom'
              disable [@action]
              enable [@uri, @method, @button, @json]
              @uri.text = @uri.text.gsub('custom/', '')
            else
              enable [@action]
              @action.items = @modules[item.text.to_sym]
            end
          end
          
        end
        
        @action = list_box width: 250, items: [''], state: 'disabled', change: 'e' do |action|
          disable [@id, @json, @button, @page]

          construct_url
          
          @method.choose 'Get'
          
          case action.text
            when :show
              enable [@id]           
            when :index
              enable [@page, @params, @button]
            when :create
              enable [@json]
              @method.choose 'Post'
            when :update
              enable [@id, @json]              
              @method.choose 'Put'
            when :delete
              enable [@id]              
              @method.choose 'Delete'
          end

        end
        
      end
      
      
      flow width: 600 do
        flow width: 100 do
          para 'UUID: ', label_style
        end
        @id = edit_line state: 'disabled', width: 500, change: 'e' do |id|
          construct_url
          
          if id.text == ''
            disable [@button]
            @datahub_checkboxes.hidden = true
          else  
            enable [@button]
            @datahub_checkboxes.hidden = false
          end
        end
      end
      
      flow width: 600 do
        flow width: 100 do
          para 'Params: ', label_style
        end
        @params = edit_line state: 'disabled', width: 500, change: 'e' do |params|
          reset_text [@response, @status]
          construct_params
        end
      end      
      
      flow width: 600 do
        flow width: 250 do
          flow width: 100 do
            para "Page: ", label_style
          end
          
          @page = edit_line state: 'disabled', width: 100, change: 'e' do |page|
            
            construct_params

          end      
        end
      end      
      
      flow width: 600 do
        flow width: 100 do
          para 'JSON: ', label_style
        end
        @json = edit_box state: 'disabled', width: 500, height: 460, change: 'e' do |json|
          reset_text [@response, @status]
          
          if json.text == ''
            disable [@button]
          else  
            enable [@button]
          end
        end
      end
      
      @button = button "Go", state: 'disabled', width: 600, height: 50 do
        valid = true
        uri = URI(@uri.text)

        http = Net::HTTP.new(uri.host, uri.port)
      
        req = Net::HTTP.const_get(@method.text).send(:new, uri)
        
        if @uri.text.include?('https')
          http.use_ssl = true      
          http.verify_mode = OpenSSL::SSL::VERIFY_NONE
        end
        
        req['Username'] = @username.text
        req['Password'] = @password.text
        req['Content-Type'] = 'application/json'
        
        # Save the current settings for next time.  The password will only be re-saved if it was originally (and manually) added to the config file.  By default the password is required every time.
        File.open('config.json', 'w') { |file| file.write(JSON.pretty_generate({username: @username.text, url: @url.text, password: DEFAULT_PASSWORD})) }
  
        if @json.text != ''
          if valid_json?(@json.text)
            req.body = JSON.parse(@json.text).to_json
          else
            alert "Invalid JSON"
            valid = false
          end
        end
        
        if valid

        
          res = http.request(req)
          @status.text = res.inspect
          @response.text = JSON.pretty_generate(JSON.parse(res.body)) rescue res.body
          
          singular_item = @items.text.gsub(/s$/, '')
          if singular_item == 'custom'          # Special cases for DataHub
            if @uri.text.include? 'message'
              singular_item = 'datahub_message'
              action = 'show'
              action = 'acknowledgement' if @uri.text.include?('acknowledgement')
              action = 'rejection' if @uri.text.include?('rejection')
              action = 'index' if @uri.text.end_with?('messages') or @uri.text.end_with?('messages/')
            else
              singular_item = 'datahub_queue'
              action = 'show'
              action = 'index' if @uri.text.end_with?('queues') or @uri.text.end_with?('queues/')
            end
          else
            action = @action.text
          end
          
          # Update the example_responses file if it exists - this is for developers to populate the documentation
          file_path = "../../config/example_responses/hr_api/v1/#{singular_item}.json"
          if File.exist?(file_path)
            contents = JSON.parse(File.read(file_path))
            
            response = JSON.parse(res.body)
            contents["#{action}"] = response

            File.open(file_path, 'w') { |file| file.write(JSON.pretty_generate(contents)) }
                        
            # Now update the schema for valid examples
            schema_file_path = "../../config/schema/hr_api/v1/#{singular_item}.json"
            if File.exist?(schema_file_path)
              schema_contents = JSON.parse(File.read(schema_file_path))
              
              response[singular_item].each_pair do |k, v|
                if v.is_a?(Hash)
                  v.each_pair do |sub_k, sub_v|
                    begin
                      schema_contents['properties']["#{k}_#{sub_k}"]['examples']['valid'] = sub_v
                    rescue
                    #  puts "No key #{k} in schema - skipping"
                    end
                  end
                else
                  begin
                    schema_contents['properties'][k]['examples']['valid'] = v
                  rescue
                  #  puts "No key #{k} in schema - skipping"
                  end
                end
              end
  
              File.open(schema_file_path, 'w') { |file| file.write(JSON.pretty_generate(schema_contents)) }
              
            end
            
            
          end
        else
        
          alert("Invalid request")
        
        end
      end
    end
  end
  
  flow width: 800, height: 840 do
    background "#efefef"
    
    stack margin: 20 do

      flow do
        flow width: 60 do
          @method = list_box width: 100, items: ['Get', 'Post', 'Put', 'Delete'], state: 'disabled'
        end
        @uri = edit_box width: 700, height: 40, text: DEFAULT_URL, state: 'disabled'
      end

      @status = edit_line width: 760, state: 'disabled'
      @response = edit_box width: 760, height: 745, state: 'readonly'
    end
  end






 end